INSERT INTO tags(name) VALUES
('audio'),
('music'),
('mac os x'),
('development'),
('reversing'),
('friends'),
('technology'),
('security'),
('p2p'),
('networking'),
('apple'),
('linux'),
('unix'),
('bsd'),
('win32'),
('art'),
('scene'),
('warez'),
('math'),
('science'),
('web'),
('xhtml'),
('css'),
('c'),
('c++'),
('objective-c'),
('perl'),
('x86'),
('Audio Units'),
('VST'),
('literature'),
('open source'),
('database'),
('video'),
('mp3'),
('divx'),
('photo'),
('paper'),
('media'),
('movie'),
('dictionary'),
('legal'),
('design'),
('magazine'),
('download'),
('news'),
('chat'),
('calendar'),
('browser')
;

