INSERT INTO tags(tagid, name) VALUES (1,'audio');
INSERT INTO tags(tagid, name) VALUES (2,'music');
INSERT INTO tags(tagid, name) VALUES (3,'mac os x');
INSERT INTO tags(tagid, name) VALUES (4,'development');
INSERT INTO tags(tagid, name) VALUES (5,'reversing');
INSERT INTO tags(tagid, name) VALUES (6,'friends');
INSERT INTO tags(tagid, name) VALUES (7,'technology');
INSERT INTO tags(tagid, name) VALUES (8,'security');
INSERT INTO tags(tagid, name) VALUES (9,'p2p');
INSERT INTO tags(tagid, name) VALUES (10,'networking');
INSERT INTO tags(tagid, name) VALUES (11,'apple');
INSERT INTO tags(tagid, name) VALUES (12,'linux');
INSERT INTO tags(tagid, name) VALUES (13,'unix');
INSERT INTO tags(tagid, name) VALUES (14,'bsd');
INSERT INTO tags(tagid, name) VALUES (15,'win32');
INSERT INTO tags(tagid, name) VALUES (16,'art');
INSERT INTO tags(tagid, name) VALUES (17,'scene');
INSERT INTO tags(tagid, name) VALUES (18,'warez');
INSERT INTO tags(tagid, name) VALUES (19,'math');
INSERT INTO tags(tagid, name) VALUES (20,'science');
INSERT INTO tags(tagid, name) VALUES (21,'web');
INSERT INTO tags(tagid, name) VALUES (22,'xhtml');
INSERT INTO tags(tagid, name) VALUES (23,'css');
INSERT INTO tags(tagid, name) VALUES (24,'c');
INSERT INTO tags(tagid, name) VALUES (25,'c++');
INSERT INTO tags(tagid, name) VALUES (26,'objective-c');
INSERT INTO tags(tagid, name) VALUES (27,'perl');
INSERT INTO tags(tagid, name) VALUES (28,'x86');
INSERT INTO tags(tagid, name) VALUES (29,'Audio Units');
INSERT INTO tags(tagid, name) VALUES (30,'VST');
INSERT INTO tags(tagid, name) VALUES (31,'literature');
INSERT INTO tags(tagid, name) VALUES (32,'open source');
INSERT INTO tags(tagid, name) VALUES (33,'database');
INSERT INTO tags(tagid, name) VALUES (34,'video');
INSERT INTO tags(tagid, name) VALUES (35,'mp3');
INSERT INTO tags(tagid, name) VALUES (36,'divx');
INSERT INTO tags(tagid, name) VALUES (37,'photo');
INSERT INTO tags(tagid, name) VALUES (38,'paper');
INSERT INTO tags(tagid, name) VALUES (39,'media');
INSERT INTO tags(tagid, name) VALUES (40,'movie');
INSERT INTO tags(tagid, name) VALUES (41,'dictionary');
INSERT INTO tags(tagid, name) VALUES (42,'legal');
INSERT INTO tags(tagid, name) VALUES (43,'design');
INSERT INTO tags(tagid, name) VALUES (44,'magazine');
INSERT INTO tags(tagid, name) VALUES (45,'download');
INSERT INTO tags(tagid, name) VALUES (46,'news');
INSERT INTO tags(tagid, name) VALUES (47,'chat');
INSERT INTO tags(tagid, name) VALUES (48,'calendar');
INSERT INTO tags(tagid, name) VALUES (49,'browser');
